#****************************************************************************************# 
# Watershed Wide R Script for Computing Confidence for Left-censored 
# Data using bootstrapping robust regression on order statistics (ROS). 
# See Helsel (2012) and Lopaka (2013) for robust ROS documentation.
# 
#    Helsel, DR. 2012. Statistics for Censored Environmental Data. Wiley 
#          and Sons.
#
#    Lopaka Lee 2013. NADA: Nondetects And Data Analysis for environmental 
#          data. R package version 1.5-5. http://CRAN.R-project.org/package=NADA
#
# This script is designed to analyze multiple groups (such as sites) and 
# parameters in one run. Specifically, bootstrapping is used to compute 
# confidence intervals for the mean.
#
# Ensure the following files are in working directory:
#    LCEB_Data.csv (or similarly formatted input data file) - data organized by
#                    columns of parameters and one sample per row. Left-censored data
#                    are entered as "<x.x" where x.x is the censoring limit. The
#                    below script reads these data and performs the necessary operations
#                    to prepare the data for the robust ROS procedure.
#    Parameters.csv (or similarly formatted parameter control file) - This file
#                    identifies the parameters to be analyzed. Included are: 
#                    parameter labels for the y- and x- axis of plots; and whether to 
#                    use log transformation for robust ROS and display of y-scale in 
#                    plots. (Also included is a group by term for directing the script
#                    to do analysis by the correct grouping--not implemented in this version) 
#
# Output: BootstrapROSCI.csv -- resulting data file of mean and confidence intervals
#                               by group and parameter
# 
# Further information:  Andrea Bolks, US-EPA Region 5, Bolks.Andrea@epa.gov
#                       Alex DeWire, Tetra Tech, Inc., Alex.DeWire@tetratech.com
#                       Jon Harcum, Tetra Tech, Inc., Jon.Harcum@tetratech.com
# Revision History:
#
#         2014-05-27 - Initial Release
#
#**********************************************************************************

#********* User updates for local use *********************************************
#
#  Enter Working Directory below (note use of "/" instead of "\")
#
WD <- "C:/Users/abolks/Documents/StatsExample"
#
#  Identify input data file and parameter control file
#
input_file <- "LCEB_Data.csv"
parm__file <- "Parameters.csv"
#
#  Enter the column name in the input file that groups parameter data for analysis 
#  (for example by site).
#
Groupby <- "Site_ID"
#
#  Enter the X label that will be used in charts. This label will be combined with the
#  actual group name. Example "Site" will produce lable such as "Site 3". 
#
Xlabel <- "Site"
#
#  Enter number of non-censored data required for generating ROS estimates. 
#  The Minimum is 3, enter a number greater if desired. 
#
nGreq  <- 3;
#
#  Enter number of bootstrap iterations 
#  
Nrep   <- 1000;
#
#  Enter percent confidence interval for CI (e.g., CI<-80 for 80% CI)
#  
CI     <- 80                # confidence interval

#********* Load all packages needed *********************************************
library("NADA"); 
library("gdata"); 
library("boot"); 
library("survival"); 
print("Will get masking messages, that's OK")

#*********  Set working directory  *********************************************
setwd(WD)
cat(getwd(), fill = TRUE, labels = "Working Directory: ");


#*********  Compute quantiles based on confidence interval (CI) ****************
#    
D_LCI  <- 0.5*(100-CI)/100; # quantile for lower confidence limit
D_MED  <- 0.5;              # median 
D_UCI  <- 1-D_LCI;          # quantile for upper confidence limit


#****************************************************************************************#
#
# Data input and QC
#
# - Reads all input data (all groups, all analytes) from a file specified by the user
# - Generate a list of parameters
#
DataTbl <- read.csv(input_file, header=TRUE);
Params <- colnames(DataTbl);
colnames(DataTbl) <- Params;

#****************************************************************************************#
#  The user selects the parameters for conducting robust ROS via the input table in 
#  "parm__file.  The complete list of possible parameters is "Params"
#  created in the code block above based on the columns of data input in the
#  data file. In the following code block "Params" is limited to the Paramaters
#  that match the parameter name in the values files.   
#
#  If the parameters input file specifies a parameter not found in the data 
#  input or a parameter appears more than once in the parameter input file,
#  execution is halted.
#
#  Note that the parameter name in the parameter input file must match the
#  parameter name in the values files EXACTLY (inlcuding periods and spaces).
#
#  The indexes of the parameters for which the script does ROS are
#  put in the array ParamROS.
#

### Read in paramater input "parm__file" and trim character strings to exclude 
### possible blank spaces in records of the csv.
ParamIn <- read.csv(parm__file, header=TRUE);
ParamIn <- trim(ParamIn);

#### Create a maxtrix from the input table reduced to the paramaters specified 
#### in the input parm__file. This drops fields not nessassary for analysis.
ParamInNam <- ParamIn$Parameter
nonParmsVec <- as.vector(setdiff(Params, ParamInNam))
Vals <- (DataTbl)
for (i in nonParmsVec){
  Vals[[i]] <- NULL
}
Vals <- as.matrix(Vals)
Params <- colnames(Vals);

#### Checks if paramaters are found or if duplicate paramaters are found. The indexes 
#### of the parameters for which the script does ROS are put in the array ParamROS.
nParamIn <- nrow(ParamIn);
ParamROS <- array(dim=c(nParamIn));
for(i in 1: nParamIn) {
  IPI <- which(as.character(Params)==as.character(ParamIn$Parameter[i]));
  if(length(IPI)==0) {
    print(list("Parameter ",as.character(ParamIn$Parameter[i])," not found"));
    stop }
  if(length(IPI)>1) {
    print(list("Parameter ",as.character(ParamIn$Parameter[i])," found more than once"));
    stop }
  ParamROS[i] <- IPI }

npROS <- length(ParamROS);

#### Generate a list of groups with data
grouplist <- unique(DataTbl[[Groupby]]);
ngroups <- length(grouplist);

#***************************************************************************************#
# Prepare data for ROS.  This entails
# - Creating a matrix of "Flags" based on 1st column of Vals
# - Setting BDL to T/F based matrix of Flags 
# - Stripping "<" from Vals 
#***************************************************************************************#

#### Create a table of flags that designate which values are <.
Flags <- substr(Vals,1,1);
Flags[Flags!="<"] <- NA;
IFlagged <- which(!is.na(Flags));
INotFlagged <- which(is.na(Flags));

#### Create the array "BDL"  this is a logical (T/F) array
#### identifying below detection limit data.
nnr <- nrow(Vals);
nnc <- ncol(Vals);
BDL <- matrix(nrow=nnr,ncol=nnc);
for(i in 1:nnr) {
   for (j in 1:nnc) {
      if(is.na(Flags[i,j])) {BDL[i,j]<-FALSE} else {
      if (Flags[i,j]=="<") {BDL[i,j]<- TRUE} else {BDL[i,j]<- FALSE} } } }
colnames(BDL) <- Params;

#### For all flagged data, strip the "<" from the record in Vals.
nflagged <- length(IFlagged);
for(i in 1:nflagged) {
   ifl <- IFlagged[i];
   Vals[ifl] <- substr(Vals[ifl],2,nchar(Vals[ifl]))}

#***************************************************************************************#
#### The following script runs bootstraps on the mean of all data provided above. 
#***************************************************************************************#

DataTbl$CG <- as.character(DataTbl[[Groupby]]);
ir <- nrow(DataTbl);
for(i in 1:ir) {
   if(nchar(DataTbl$CG[i])<2) { DataTbl$CG[i] <- paste("0",DataTbl$CG[i],sep="") } } 
DataTbl$CG[nchar(DataTbl$CG)<2] <- paste("0",DataTbl,sep="");
DataTbl$CG <- paste("Group"),DataTbl$CG);

mean.ros <- function(GG){
  V <- GG[,1];
  B <- GG[,2];
  x <- try(mean(ros(V,B)))
  return(x)} 


mean.datanow <- function(GG){
   V <- GG[,1];
   B <- GG[,2];
   nG <- length(V[which(!B)]);
   nB <- length(which(B));
   DD <- 10;
   if(nB > 0 & nG > 0) {DD <- mean(V[which(!B)]) - mean(V[which(B)]) }
   if(nG>2 & DD > 0) {
      y <- try(mean(ros(V,B))) } else
   {  y <- as.numeric(NA) }
   return(y)}

SSX <- NULL;         # Matrix for storing results for all groups, parameters
for(k in 1:npROS) {  # Loop for parameters
   pp <- ParamROS[k];
   pylab <- ParamIn$Label[k];
   print(pylab);
   pplog <- ParamIn$Scale[k];
   uselog <- FALSE
   if(as.character(pplog) == "Log") {uselog=TRUE}

   nsp1 <- ngroups+1;

   Pname <- array(dim=nsp1);
   Sname <- array(dim=nsp1);
   nD <- array(dim=nsp1);
   nBDL <- array(dim=nsp1);
   maxDL <- array(dim=nsp1);
   maxV <- array(dim=nsp1);
   nG <- array(dim=nsp1);
   GoodSite <- array(dim=nsp1);
   DMean <- array(dim=nsp1);
   LCI <- array(dim=nsp1);
   MED <- array(dim=nsp1);
   UCI <- array(dim=nsp1);

   VTemp <- as.numeric(Vals[,Params[pp]]);
   BTemp <- BDL[,Params[pp]];
   STemp <- DataTbl$CG;
   
   for (m in 1:ngroups) {      # Loop for groups
      js <- which(DataTbl[[Groupby]] == grouplist[m] & !is.na(VTemp));
      TV <- VTemp[js];
      TB <- BTemp[js];
      TS <- STemp[js];
      Sname[m] <- TS[1];
      Pname[m] <- as.character(pylab);
      print(TS[1]);
      iBDL <- which(TB);
      nBDL[m] <- length(iBDL);   # No. of BDL data for the group & param
      maxDL[m] <- -1;
      if(nBDL[m] > 0) { maxDL[m] <- max(TV[iBDL]) }
      nD[m] <- length(TV); # No of data
      maxV[m] <- max(TV);
      nG[m] <- nD[m] - nBDL[m]; # No. non-censored data for the group/param
      GoodSite[m] <- FALSE
      if(nG[m] >= nGreq){
        GG <- data.frame(TV,TB);
        DMean[m] <- mean.ros(GG)}
      if(nG[m] > 5) {
         GoodSite[m] <- TRUE 
         GG <- data.frame(TV,TB);
         censoredbootstrap <- try(censboot(GG, mean.datanow, R=Nrep))
         LCI[m] <- quantile(censoredbootstrap$t,D_LCI,na.rm=TRUE);
         MED[m] <- quantile(censoredbootstrap$t,D_MED,na.rm=TRUE);
         UCI[m] <- quantile(censoredbootstrap$t,D_UCI,na.rm=TRUE) } }

   js <- which(!is.na(VTemp));
   TV <- VTemp[js];
   TB <- BTemp[js];
   TS <- STemp[js];
   Sname[nsp1] <- "Watershed";
   Pname[nsp1] <- as.character(pylab);
   iBDL <- which(TB);
   nBDL[nsp1] <- length(iBDL);   # No of BDL data for the group & param
   maxDL[nsp1] <- -1;
   if(nBDL[nsp1] > 0) { maxDL[nsp1] <- max(TV[iBDL]) }
   nD[nsp1] <- length(TV); # No of data
   maxV[nsp1] <- max(TV);
   nG[nsp1] <- nD[nsp1] - nBDL[nsp1]; # No. non-censored data for the group/param
   GoodSite[nsp1] <- FALSE
   if(nG[nsp1] >= nGreq){
     GG <- data.frame(TV,TB);
     DMean[nsp1] <- mean.ros(GG)}
   if(nG[nsp1] > 5) {
      GoodSite[nsp1] <- TRUE 
      GG <- data.frame(TV,TB);
      censoredbootstrap <- try(censboot(GG, mean.datanow, R=Nrep));
      LCI[nsp1] <- quantile(censoredbootstrap$t,D_LCI,na.rm=TRUE);
      MED[nsp1] <- quantile(censoredbootstrap$t,D_MED,na.rm=TRUE);
      UCI[nsp1] <- quantile(censoredbootstrap$t,D_UCI,na.rm=TRUE) }

   TDF <- data.frame(Pname,Sname,nD,nBDL,nG,DMean,LCI,MED,UCI);
  colnames(TDF) <- c("Parameter","Group","No. data","No. censored",
                     "No. uncensored","Mean", paste("Lower", CI,"%CI"),
                     "Median", paste("Upper", CI,"%CI"));
   SSX <- rbind(SSX,TDF);

} # End of parameters loop

write.csv(SSX,file="BootstrapROSCI.csv");
