#****************************************************************************************# 
# Watershed Wide R Script for Computing Summary Statistics for Left-censored 
# Data using robust regression on order statistics (ROS). See Helsel (2012) and
# Lopaka (2013) for robust ROS documentation.
# 
#    Helsel, DR. 2012. Statistics for Censored Environmental Data. Wiley 
#          and Sons.
#
#    Lopaka Lee 2013. NADA: Nondetects And Data Analysis for environmental 
#          data. R package version 1.5-6. http://CRAN.R-project.org/package=NADA
#
# This script is designed to analyze multiple groups (such as sites) and 
# parameters in one run. For each parameter one figure is created with one 
# censored boxplot per group. Maximum censored value is shown by shading for each
# group independently. Probability plots are created for each parameter-group 
# combination and stored in separate subdirectories. Analyst should review these
# plots to ensure that the selected normal or lognormal distribution is adequate.
# Script will work on left-censored data with multiple censoring levels and
# uncensored data sets as well. 
#
# Ensure the following files are in working directory:
#    cenboxplot2.R - modified version of cenboxplot that leaves a space for groups
#                    with insufficient data for analysis
#    LCEB_Data.csv (or similarly formatted input data file) - data organized by
#                    columns of parameters and one sample per row. Left-censored data
#                    are entered as "<x.x" where x.x is the censoring limit. The
#                    below script reads these data and performs the necessary operations
#                    to prepare the data for the robust ROS procedure.
#    Parameters.csv (or similarly formatted parameter control file) - This file
#                    identifies the parameters to be analyzed. Included are: 
#                    parameter labels for the y- and x- axis of plots; and whether to 
#                    use log transformation for robust ROS and display of y-scale in 
#                    plots. (Also included is a group by term for directing the script
#                    to do analysis by the correct grouping--not implemented in this version) 
#
# Output: ROSSummaryStats.csv -- resulting data file of summary statistics by group and 
#                                parameter
# 
# Further information:  Andrea Bolks, US-EPA Region 5, Bolks.Andrea@epa.gov
#                       Alex DeWire, Tetra Tech, Inc., Alex.DeWire@tetratech.com
#                       Jon Harcum, Tetra Tech, Inc., Jon.Harcum@tetratech.com
# Revision History:
#
#         2014-05-27 - Initial Release
#         2016-07-17 - Corrected boxplot maximum DL shading when log transformations
#                      are not used.
#
#**********************************************************************************

#********* User updates for local use *********************************************
#
#  Enter Working Directory below (note use of "/" instead of "\")
#
WD <- "C:/Users/abolks/Documents/StatsExample"
#
#  Identify input data file and parameter control file
#
input_file <- "LCEB_Data.csv"
parm__file <- "Parameters.csv"
#
#  Enter the column name in the input file that groups parameter data for analysis 
#  (for example by site).
#
Groupby <- "Site_ID"
#
#  Enter the X label that will be used in charts. This label will be combined with the
#  actual group name. Example "Site" will produce lable such as "Site 3". 
#
Xlabel <- "Site"
#
#  Enter number of non-censored data required for generating ROS estimates. 
#  The minimum is 3, enter a number greater if desired. 
#
nGreq  <- 3;

#********* Load all packages needed *********************************************
library("NADA") 
library("gdata")  
print("Will get masking messages, that's OK")

#*********  Set working directory  *********************************************
setwd(WD)
cat(getwd(), fill = TRUE, labels = "Working Directory: ");

#*****************************************************************************#
# Source function for producing boxplots of censored data (using robust ROS).
# This function is a modification of the function "cenboxplot" from the
# NADA package.  See comments at the top of the function for details on
# the modifications.  As currently programmed, the file "cenboxplot2.R"
# must reside in the working directory.

source("cenboxplot2.R")

#****************************************************************************************#
#
# Data input and QC
#
# - Reads all input data (all groups, all analytes) from a file specified by the user
# - Generate a list of parameters
#
DataTbl <- read.csv(input_file, header=TRUE);
Params <- colnames(DataTbl);
colnames(DataTbl) <- Params;

#****************************************************************************************#
#  The user selects the parameters for conducting robust ROS via the input table in 
#  "parm__file.  The complete list of possible parameters is "Params"
#  created in the code block above based on the columns of data input in the
#  data file. In the following code block "Params" is limited to the Paramaters
#  that match the parameter name in the values files.   
#
#  If the parameters input file specifies a parameter not found in the data 
#  input or a parameter appears more than once in the parameter input file,
#  execution is halted.
#
#  Note that the parameter name in the parameter input file must match the
#  parameter name in the values files EXACTLY (inlcuding periods and spaces).
#
#  The indexes of the parameters for which the script does ROS are
#  put in the array ParamROS.
#

### Read in paramater input "parm__file" and trim character strings to exclude 
### possible blank spaces in records of the csv.
ParamIn <- read.csv(parm__file, header=TRUE);
ParamIn <- trim(ParamIn);

#### Create a maxtrix from the input table reduced to the paramaters specified 
#### in the input parm__file. This drops fields not nessassary for analysis.
ParamInNam <- ParamIn$Parameter
nonParmsVec <- as.vector(setdiff(Params, ParamInNam))
Vals <- (DataTbl)
for (i in nonParmsVec){
  Vals[[i]] <- NULL
}
Vals <- as.matrix(Vals)
Params <- colnames(Vals);

#### Checks if paramaters are found or if duplicate paramaters are found. The indexes 
#### of the parameters for which the script does ROS are put in the array ParamROS.
nParamIn <- nrow(ParamIn);
ParamROS <- array(dim=c(nParamIn));
for(i in 1: nParamIn) {
  IPI <- which(as.character(Params)==as.character(ParamIn$Parameter[i]));
  if(length(IPI)==0) {
    print(list("Parameter ",as.character(ParamIn$Parameter[i])," not found"));
    stop }
  if(length(IPI)>1) {
    print(list("Parameter ",as.character(ParamIn$Parameter[i])," found more than once"));
    stop }
  ParamROS[i] <- IPI }

npROS <- length(ParamROS);

#### Generate a list of groups with data
grouplist <- unique(DataTbl[[Groupby]]);
ngroups <- length(grouplist);

#***************************************************************************************#
# Prepare data for ROS.  This entails
# - Creating a matrix of "Flags" based on 1st column of Vals
# - Setting BDL to T/F based matrix of Flags 
# - Stripping "<" from Vals 
#***************************************************************************************#

#### Create a table of flags that designate which values are <.
Flags <- substr(Vals,1,1);
Flags[Flags!="<"] <- NA;
IFlagged <- which(!is.na(Flags));
INotFlagged <- which(is.na(Flags));

#### Create the array "BDL"  this is a logical (T/F) array
#### identifying below detection limit data.
nnr <- nrow(Vals);
nnc <- ncol(Vals);
BDL <- matrix(nrow=nnr,ncol=nnc);
for(i in 1:nnr) {
   for (j in 1:nnc) {
      if(is.na(Flags[i,j])) {BDL[i,j]<-FALSE} else {
      if (Flags[i,j]=="<") {BDL[i,j]<- TRUE} else {BDL[i,j]<- FALSE} } } }
colnames(BDL) <- Params;

#### For all flagged data, strip the "<" from the record in Vals.
nflagged <- length(IFlagged);
for(i in 1:nflagged) {
   ifl <- IFlagged[i];
   Vals[ifl] <- substr(Vals[ifl],2,nchar(Vals[ifl]))}
   
#***************************************************************************************#
#### Creates an output of summary statistics, 25th and 75th percentiles and
#### an overall summary in a watershed wide (batch) output file. 
#### This script can be used for non-censored data as well.
#### Other data that might be usful include:
####
#### The loops below loop through all of the parameters of interest (found
#### in the vector ParamROS) and all of the groups (in the vector grouplist) plus
#### watershed wide (non-grouped) calculations.
####
#### For each parameter-group pair, ROS is used to estimate the distribution
#### 25th percetnile, median, mean and 75th percentile value.  these are 
#### output to a csv file along with parameter, group, of observations and 
#### # of censored observations.  If the number of non-censored data is less 
#### than nGreq, no calculations are done and the results for that group-parameter
#### pair are "NA".  
####
#### Results are output as a single table to file ROSSummaryStats.csv.
#### csv format was chosen for easy use with Excel.
#***************************************************************************************#
mainDir <- getwd();

SSX <- NULL;         # Matrix for storing results for all groups, parameters
for(k in 1:npROS) {  # Loop for parameters
   pp <- ParamROS[k];
   pylab <- ParamIn$Label[k];
   subDir <- as.character(Params[pp]);
   if (file.exists(subDir)){
      setwd(file.path(mainDir, subDir))
   } else {
      dir.create(file.path(mainDir, subDir))
      setwd(file.path(mainDir, subDir)) }
   SumStats <- matrix(ncol=7,nrow=ngroups+1);  # Summary stats
   SiteN <- paste(Xlabel,grouplist);
   SiteN <- append(SiteN,"Watershed");
   rownames(SumStats) <- SiteN
   colnames(SumStats) <- c("Parameter","No. obs","No. cens",
        "1st quantile","Median","Mean","3rd quantile");
   for(isite in 1:ngroups) {    # Loop for groups
      SumStats[isite,1] <- Params[pp];
      iss <- grouplist[isite]
      is <- which(DataTbl[[Groupby]]==iss)
      Inna <- which(!is.na(as.numeric(Vals[,pp])) & DataTbl[[Groupby]]==iss);
      V <- as.numeric(Vals[Inna,pp]);
      nV <- length(V);
      C <- BDL[Inna,pp];
      nC <- length(which(C));
      nG <- nV - nC;
      SumStats[isite,2] <- nV;
      SumStats[isite,3] <- nC;
      if(nG >= nGreq) {
         X <- try(ros(V,C),silent=TRUE)
         SumStats[isite,4] <- quantile(X,0.25);
         SumStats[isite,5] <- quantile(X,0.5);
         SumStats[isite,6] <- mean(X);
         SumStats[isite,7] <- quantile(X,0.75);
         cname <- paste(as.character(Params[pp]),Xlabel,sep=" ");
         cname <- paste(cname,as.character(iss));
         fname <- paste("ExceedancePlot",cname);
         fname <- paste(fname,".jpg");
         jpeg(file = fname,width=5,height=5,res=300,units="in");
         try(plot(X, ylab=pylab, plot.censored=TRUE,cex.lab=1.2))
         dev.off() } } 

   Inna <- which(!is.na(as.numeric(Vals[,pp])));
   V <- as.numeric(Vals[Inna,pp]);
   nV <- length(V);
   C <- BDL[Inna,pp];
   nC <- length(which(C));
   nG <- nV - nC;
   SumStats[ngroups+1,1] <- Params[pp];
   SumStats[ngroups+1,2] <- nV;
   SumStats[ngroups+1,3] <- nC;
   if(nG >= nGreq) {
      X <- try(ros(V,C),silent=TRUE)
      SumStats[ngroups+1,4] <- quantile(X,0.25);
      SumStats[ngroups+1,5] <- quantile(X,0.5);
      SumStats[ngroups+1,6] <- mean(X);
      SumStats[ngroups+1,7] <- quantile(X,0.75) }
   SSX <- rbind(SSX,SumStats) }

setwd(file.path(mainDir))
fname <- "ROSSummaryStats.csv"
write.csv(SSX,file=fname);

#***************************************************************************************#
#### Creates boxplots for all groups and parameters.  One boxplot per parameter,
#### with all groups on the same boxplot. A new field had to be created to
#### cause groups to appear in the correct order on boxplots.
####
#### Results are output as a single table to file ROSSummaryStats.csv.
#### csv format was chosen for easy use with Excel.
#***************************************************************************************#

DataTbl$CG <- as.character(DataTbl[[Groupby]]);
ir <- nrow(DataTbl);
for(i in 1:ir) {
   if(nchar(DataTbl$CG[i])<2) { DataTbl$CG[i] <- paste("0",DataTbl$CG[i],sep="") } } 
DataTbl$CG[nchar(DataTbl$CG)<2] <- paste("0",DataTbl,sep="");
DataTbl$CG <- paste(Xlabel, " ",DataTbl$CG);

#### Create the boxplots - one plot per parameter, all groups shown

for(k in 1:npROS) {  # Loop for parameters
   pp <- ParamROS[k];
   pylab <- ParamIn$Label[k];
   print(pylab);
   pplog <- ParamIn$Scale[k];
   uselog <- FALSE
   if(as.character(pplog) == "Log") {uselog=TRUE}
   fname <- paste("Boxplot",Params[pp],sep="");
   fname <- paste(fname,".jpg",sep="");

   VTemp <- as.numeric(Vals[,Params[pp]]);
   BTemp <- BDL[,Params[pp]];
   STemp <- DataTbl$CG;

   nD <- array(dim=ngroups);
   nBDL <- array(dim=ngroups);
   maxDL <- array(dim=ngroups);
   maxV <- array(dim=ngroups);
   nG <- array(dim=ngroups);
   GoodSite <- array(dim=ngroups);

   for (m in 1:ngroups) {
      js <- which(DataTbl[[Groupby]] == grouplist[m]);
      iBDL <- which(BDL[js,pp]);
      nBDL[m] <- length(iBDL);   # No of BDL data for the group & param
      maxDL[m] <- -1;
      if(nBDL[m] > 0) { maxDL[m] <- max(Vals[iBDL,pp]) }
      nD[m] <- length(!is.na(as.numeric(Vals[js,pp]))); # No of data
      imV <- which.max(!is.na(as.numeric(Vals[js,pp])));
      maxV[m] <- max(as.numeric(Vals[js,pp]),na.rm=TRUE);
      nG[m] <- nD[m] - nBDL[m]; # No non-censored data for the group/param
      GoodSite[m] <- FALSE
      if(nG[m] > 5) {GoodSite[m] <- TRUE 
         idat <- which(!is.na(as.numeric(Vals[js,pp]))) } }

   maxDL <- as.numeric(maxDL);

   jpeg(file=fname,height=5,width=8.5,res=450,units="in");
   par(mai=c(1.1,0.9,0.2,0.2));
   ibp <- which(!is.na(VTemp));
   cenboxplot2(VTemp[ibp],BTemp[ibp],STemp[ibp], range=1.5, na.rm=TRUE,
       log=uselog, las=2, ylab=pylab, cex.lab=1.2, cex.axis=0.8,boxwex=0.5,
       nGreq=nGreq);
   LowY <- ifelse(uselog, 10^(par("usr")[3]) , (par("usr")[3]) ) ;
   for (m in 1:ngroups) {
      if(GoodSite[m]) {
         if(nBDL[m]>0) {
            ii <- which(DataTbl[[Groupby]] == grouplist[m] & BTemp);
            maxDLL <- max(VTemp[ii]);
            xpol <- c((m-0.25),(m+0.25),(m+0.25),(m-0.25));
            ypol <- c(maxDLL,maxDLL,LowY,LowY);
            tt <- paste("n=",nG[m]);
            par(srt=90);
            polygon(xpol,ypol,col=rgb(0.75,0.75,0.75,0.5),
                border=NA) } } }
   dev.off() }