#**********************************************************************#
# Revised version of cenboxplot.  
#
# The unmodified function cenboxplot creates a boxplot of the 
# modeled data for a censored ROS object, with one box per data
# group (i.e., per site in the Region V data set).
# 
# Revisions to this version avoid errors related to groups that have 
# too few data because there are NAs in the data set or that have 
# only censored data.  In both cases the modeled data for that group
# are replaced by NA, which causes the boxplot routine to skip making
# a box for that site.
#
# The number of non-censored observations required for 
# generating a box for a group is set as the variable nGreq in the
# first line of the function.  The minimum value that allows the 
# function to run for nGreq is 2 (i.e., there must be > 2 observations 
# for the ROS to be performed).  This is the value that allows the 
# routine to run, though boxplots with few observations could 
# lead to unusual results.
#
# Revision History:
#
#         2014-05-27 - Initial Release
#
#**********************************************************************#

cenboxplot2 <- function (obs, cen, group, log = TRUE, range = 0, 
                         nGreq = 2, ...) 
{
    if (log) 
        log = "y"
    else log = ""
    if (missing(group)) 
        ret = boxplot(cenros(obs, cen), log = log, range = range, 
            ...)
    else {
        modeled = numeric()
        groups = character()
        for (i in levels(as.factor(group))) {
            obsx <- obs[group==i & !is.na(obs)];
            cenx <- cen[group==i & !is.na(obs)];
            nG <- length(obsx) - length(which(cenx));
            if(nG > nGreq) {
               mod = suppressWarnings(cenros(obsx,cenx)$modeled) } else
            {mod <- rep(NA,length(obsx)) }
            grp = rep(i, length(mod))
            modeled = c(modeled, mod)
            groups = c(groups, grp)
        }
        boxplot(modeled ~ as.factor(groups), log = log, range = range, 
            ...)
        ret = data.frame(ros.model = modeled, group = groups)
    }
    invisible(ret)
}
         


