#****************************************************************************************# 
# Example script for computing summary statistics for left-censored 
# data using robust regression on order statistics (ROS), maximum
# likelihood estimation (MLE), and Kaplan-Meier (KM) method. See 
# Helsel (2012) and Lopaka (2013) for documentation.
# 
#    Helsel, DR. 2012. Statistics for Censored Environmental Data. Wiley 
#          and Sons.
#
#    Lopaka Lee 2013. NADA: Nondetects And Data Analysis for environmental 
#          data. R package version 1.5-5. http://CRAN.R-project.org/package=NADA
#
# This script is designed to a analyze single groups (such as a site). 
#
# last revised: 5/27/2014
# -------------------------------------------------------------
# -- load R packages and set working directory      Example 1a
#
setwd("C:/Users/abolks/Documents/StatsExample")
library(NADA)
library(Rcmdr, quietly=TRUE);  closeCommander(ask=FALSE)

# -------------------------------------------------------------
# -- import example data file and summarize total   Example 1b
#    phosphorus data using detection limit substitution
#
example1 <- read.csv("Site43.csv")
summary(example1$TP)
View(example1)

# -------------------------------------------------------------
# -- attach example 1; compute summary statistics;  Example 1c
#    prepare boxplots and various x-y bivariate plots
#
attach(example1)
summary(TP) 
numSummary(TP, statistics=c("mean", "sd", "IQR",  
   "quantiles", "cv"), quantiles=c(0,.25,.5,.75,1))
boxplot(TP~Season, log="y", range=1.5, xlab="Season", 
   ylab="TP, mg/L")
plot(TP~TSS, log="xy", xlab="TSS, mg/L", ylab="TP, mg/L")
scatterplot(TP~TSS | TP_Cen, log="xy", reg.line=lm, 
   by.groups=FALSE, xlab="TSS, mg/L", ylab="TP, mg/L",
   legend.title="TP Censoring", legend.coords="bottomright", 
   boxplots="xy", smoother=FALSE, cex=1.5)

# -------------------------------------------------------------
# -- perform robust ROS assuming LOGNORMAL distribution  Example 1d
#    for TP data; graph probability plot for uncensored 
#    data print out summary statistics 
#
# -- For normal distribution, substitute the following
#    two lines for the corresponding lines:
#    ros_res = ros(TP, TP_Cen, forwardT=NULL, reverseT=NULL) 
#    plot(ros_res, sub="Normal--no transformation",plot.censored=TRUE)
#
ros_res = ros(TP, TP_Cen, forwardT="log", reverseT="exp") 
plot(ros_res, sub="Lognormal transformation",plot.censored=TRUE)
summary(ros_res)
median(ros_res); mean(ros_res); sd(ros_res) 
quantile(ros_res)

# -------------------------------------------------------------
# -- compute MLE assuming LOGNORMAL distribution    Example 1e
#    with 90% confidence interval on mean
#
# -- for normal distribution, substitute the following
#    lines; omitting conf.int yields a default 95% CI: 
#    mle_res = cenmle(TP,TP_Cen, dist="gaussian")
#    plot(mle_res, sub="Normal--no transformation")
#
mle_res = cenmle(TP,TP_Cen, dist="lognormal", conf.int=0.90)
plot(mle_res, sub="Lognormal transformation")
summary(mle_res) 
median(mle_res); mean(mle_res); sd(mle_res) 
quantile(mle_res)

# -------------------------------------------------------------
# -- compute Kaplan-Meier                           Example 1f
#    with 90% confidence interval on mean
#    omitting conf.int yields a default 95% CI 
#
#
km_res = cenfit(TP,TP_Cen, conf.int=0.90)
plot(km_res)
summary(km_res) 
median(km_res); mean(km_res); sd(km_res) 
quantile(km_res)

# --- bonus code ----------------------------------------------
# -------------------------------------------------------------
# -- compare robust MLE to robust ROS results;      Example 1g
#    also include includes Kaplan Meier results 
#
censtats(TP, TP_Cen)
